/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


#ifndef DIMPRO_FREQUENCY_HPP_
#define DIMPRO_FREQUENCY_HPP_

#include "scalar.hpp"

namespace NSDimensionPromise
{

/**
 * Frequencyの次元定義.
 */
template <typename T>
struct FrequencyBaseTraits
{
    typedef ScalarNumber<T, 0, 0, -1, 0, 0, 0, 0> ScalarType;
};

/**
 * Frequencyを表すベースクラス.
 */
template <typename T>
class FrequencyBase : public FrequencyBaseTraits<T>::ScalarType
{
public:
    /**
     * Frequencyを表すスカラー変数を生成する.
     * @param hz 変数の初期値
     */
    explicit FrequencyBase(const T& hz = T())
        : FrequencyBaseTraits<T>::ScalarType(hz)
    {  }

    /**
     * Frequencyを表すスカラー変数をコピーして生成する.
     * @param rhs コピー元の変数
     */
    FrequencyBase(const typename FrequencyBaseTraits<T>::ScalarType& rhs)
        : FrequencyBaseTraits<T>::ScalarType(rhs)
    {  }

    /**
      * Hz単位の値を得る.
      * @return 周波数
      */
    const T& ToHz() const
    {
        return this->X();
    }

    /**
      * HzからFrequencyを表すスカラー変数を生成する.
      * @return Frequencyを表すスカラー変数
      */
    static FrequencyBase FromHz(const T& hz)
    {
        return FrequencyBase(hz);
    }
};

/**
 * よく使うFrequencyの型.
 */
typedef FrequencyBase<NormalValueType> Frequency;

}

#endif /* DIMPRO_FREQUENCY_HPP_ */
